/******************************************************************************
 * Copyright (c) 2013, Vladimir Kravets                                       *
 * All rights reserved.                                                       *
 *                                                                            *
 * Redistribution and use in source and binary forms, with or without         *
 * modification, are permitted provided that the following conditions are     *
 * met: Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer.                      *
 * Redistributions in binary form must reproduce the above copyright notice,  *
 * this list of conditions and the following disclaimer in the documentation  *
 * and/or other materials provided with the distribution.                     *
 * Neither the name of the Fido4Java nor the names of its contributors        *
 * may be used to endorse or promote products derived from this software      *
 * without specific prior written permission.                                 *
 *                                                                            *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,      *
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR     *
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR          *
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,      *
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,        *
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; 
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,   *
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    *
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,             *
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                         *
 ******************************************************************************/

package org.fidonet.misc;

import org.fidonet.logger.ILogger;
import org.fidonet.logger.LoggerFactory;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.Enumeration;
import java.util.LinkedList;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class Zipper {

    private static final ILogger logger = LoggerFactory.getLogger(Zipper.class.getName());

    public static LinkedList<PktTemp> unpackboundlfast(String bound) throws IOException {
        logger.info("Unpack and toss: " + bound);
        LinkedList<PktTemp> res = new LinkedList<PktTemp>();

        ZipFile zip = new ZipFile(bound);

        Enumeration entries = zip.entries();

        byte[] buf = new byte[1024];

        while (entries.hasMoreElements()) {
            ZipEntry entry = (ZipEntry) entries.nextElement();

            if (entry.isDirectory()) {
                //(new File(entry.getName())).mkdir();
                continue;
            }

            logger.debug("        " + entry.getName());

            ByteBuffer resbuf = ByteBuffer.allocate((int) entry.getSize());
            resbuf.order(ByteOrder.LITTLE_ENDIAN);
            resbuf.position(0);
            InputStream fis = zip.getInputStream(entry);
            while (fis.available() > 0) {
                int readed = fis.read(buf);
                resbuf.put(buf, 0, readed);
            }
            resbuf.position(0);
            PktTemp pkt = new PktTemp(entry.getName(), resbuf);
            res.push(pkt);
        }
        zip.close();
        logger.info("Unpacking was finished");
        return res;
    }

}
